Files
TheRock/rocm-libraries/projects/rocsolver/clients/common/misc/rocblas_test.hpp
T
derek.holloway 7502bf64a4
CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
Flatten subprojects into a single base monorepo
2026-09-02 16:34:47 -07:00

168 lines
7.7 KiB
C++

/* **************************************************************************
* Copyright (C) 2018-2026 Advanced Micro Devices, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
*
* 1. Redistributions of source code must retain the above copyright
* notice, this list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright
* notice, this list of conditions and the following disclaimer in the
* documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
* FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
* DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
* OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
* LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
* OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
* SUCH DAMAGE.
* *************************************************************************/
#pragma once
#include <algorithm>
#include <cstdio>
#include <cstring>
#include <functional>
#include <iostream>
#include <sstream>
#include <string>
#include <type_traits>
#include <unordered_map>
#include <utility>
#include <fmt/core.h>
#include <fmt/ostream.h>
#include <rocblas/rocblas.h>
// Suppress warnings about hipMalloc(), hipFree() except in rocblas-test and
// rocblas-bench
#if !defined(ROCSOLVER_CLIENTS_TEST) && !defined(ROCBLAS_BENCH)
#undef hipMalloc
#undef hipFree
#endif
#ifdef ROCSOLVER_CLIENTS_TEST
#include <gtest/gtest.h>
// Extra macro so that macro arguments get expanded before calling Google Test
#define CHECK_HIP_ERROR2(ERROR) ASSERT_EQ(ERROR, hipSuccess)
#define CHECK_HIP_ERROR(ERROR) CHECK_HIP_ERROR2(ERROR)
#define CHECK_DEVICE_ALLOCATION(ERROR) \
do \
{ \
auto error = ERROR; \
if(error == hipErrorOutOfMemory) \
{ \
/* Reset hip error code */ \
error = hipGetLastError(); \
GTEST_SKIP() << LIMITED_MEMORY_STRING; \
return; \
} \
else if(error != hipSuccess) \
{ \
FAIL() << hipGetErrorString(error); \
return; \
} \
} while(0)
#define CHECK_ROCBLAS_ALLOCATION(ERROR) \
do \
{ \
auto error = ERROR; \
if(error == rocblas_status_memory_error) \
{ \
GTEST_SKIP() << LIMITED_MEMORY_STRING; \
return; \
} \
else if(error != rocblas_status_success) \
{ \
FAIL() << rocblas_status_to_string(error); \
return; \
} \
} while(0)
#define CHECK_ALLOC_QUERY(STATUS) \
do \
{ \
auto status__ = (STATUS); \
ASSERT_TRUE(status__ == rocblas_status_size_increased \
|| status__ == rocblas_status_size_unchanged); \
} while(0)
#define EXPECT_ROCBLAS_STATUS ASSERT_EQ
#else // ROCSOLVER_CLIENTS_TEST
inline void rocblas_expect_status(rocblas_status status, rocblas_status expect)
{
if(status != expect)
{
fmt::print(stderr, "rocBLAS status error: Expected {}, received {}\n",
rocblas_status_to_string(expect), rocblas_status_to_string(status));
if(expect == rocblas_status_success)
exit(EXIT_FAILURE);
}
}
#define CHECK_HIP_ERROR(ERROR) \
do \
{ \
auto error = ERROR; \
if(error != hipSuccess) \
{ \
fmt::print(stderr, "error: {} ({}) at {}:{}\n", hipGetErrorString(error), \
static_cast<int32_t>(error), __FILE__, __LINE__); \
rocblas_abort(); \
} \
} while(0)
#define CHECK_ALLOC_QUERY(STATUS) \
do \
{ \
auto status__ = (STATUS); \
if(!(status__ == rocblas_status_size_increased || status__ == rocblas_status_size_unchanged)) \
{ \
fmt::print(stderr, \
"rocBLAS status error: Expected rocblas_status_size_unchanged or " \
"rocblas_status_size_increase,\nreceived {}\n", \
rocblas_status_to_string(status__)); \
rocblas_abort(); \
} \
} while(0)
#define CHECK_DEVICE_ALLOCATION CHECK_HIP_ERROR
#define EXPECT_ROCBLAS_STATUS rocblas_expect_status
// The info provided to EXPECT macros is used in rocsolver-test, but
// in rocsolver-bench, the information is just discarded.
struct rocsolver_info_discarder
{
template <typename T>
rocsolver_info_discarder& operator<<(T&&)
{
return *this;
}
};
#define EXPECT_EQ(v1, v2) rocsolver_info_discarder()
#define EXPECT_NE(v1, v2) rocsolver_info_discarder()
#define EXPECT_LT(v1, v2) rocsolver_info_discarder()
#define EXPECT_LE(v1, v2) rocsolver_info_discarder()
#define EXPECT_GT(v1, v2) rocsolver_info_discarder()
#define EXPECT_GE(v1, v2) rocsolver_info_discarder()
#endif // ROCSOLVER_CLIENTS_TEST
#define CHECK_ROCBLAS_ERROR2(STATUS) EXPECT_ROCBLAS_STATUS(STATUS, rocblas_status_success)
#define CHECK_ROCBLAS_ERROR(STATUS) CHECK_ROCBLAS_ERROR2(STATUS)